<?php

class Settings extends Cf_Controller
{

    function __construct()
    {
        parent::__construct();

        setCurrentApplication('settings');

        $this->load->library('form_validation');
        $this->load->library('ion_auth');
        $this->load->library('ldap_api');
        $this->load->helper('form');
        $this->load->helper('designcenter/dc_path');
        $this->load->helper(array(
            'url',
            'cf_directory_helper'
        ));
        $this->load->model('settings_rest_model');
        $this->settings_rest_model->setRestClient($this->getRestClient());
    }

    function index()
    {
        redirect('settings/manage'); //default action
    }

    function manage()
    {
        $requiredjs = [['widgets/notes.js']];
        $this->carabiner->js($requiredjs);

        if (!$this->ion_auth->is_accessible()) {
            redirect('auth/view_profile');
        }

        try {
            $settingsValue = $this->settings_rest_model->get_app_settings();
            unset($settingsValue['ldapPasswordIsSet']);
        } catch (Exception $e) {
            // for now set the setting value as empty array whn error
            show_error($e->getMessage());
        }

        $this->form_validation->set_rules('mode', 'Authentication mode', 'xss_clean|trim');
        $this->form_validation->set_rules('rbac', 'Role based access control', 'xss_clean|trim|required');
        $this->form_validation->set_rules('bluehost_threshold_global', 'Blue host horizon', 'callback_validate_bluehost_threshold');
        $this->form_validation->set_rules('sameHostsNumberOfRuns', 'sameHostsNumberOfRuns', 'xss_clean|trim|required|greater_than[1]');
        $this->form_validation->set_error_delimiters('<span>', '</span><br/>');

        if ($this->form_validation->run($this) == true) {
            log_message('debug', "manage(): Mission Portal validation - OK");

                $returnData = $this->_updateData($settingsValue); //pass OLD data to compare new and old pass
                if (is_array($returnData) && $returnData['status']) {
                    $this->session->set_flashdata('message', array(
                        'content' => $returnData['message'],
                        'type' => 'alert-success'
                    ));
                    redirect('settings/manage');
                } else {
                    $feedback = array(
                        'type' => 'alert-error',
                        'content' => $returnData['message']
                    );
                }


        } else {
            log_message('debug', "manage(): Mission Portal validation - Failed");
            $feedback = validation_errors();
        }


        $bluehost_threshold_min = set_value('bluehost_threshold_global', $settingsValue['blueHostHorizon'] / 60);
        $mode = $settingsValue['ldapEnabled'] ? 'ldap' : 'internal';

        $form_data = array(
            'appemail' => set_value('appemail', $settingsValue['appemail']),
            'mode' => set_value('mode', $mode),
            'logLevel' => $settingsValue['logLevel'],
            'rbac' => set_value('rbac', $settingsValue['rbacEnabled']),
            'bluehost_threshold_global' => set_value('bluehost_threshold_global', $bluehost_threshold_min),
            'sameHostsNumberOfRuns'     => $settingsValue['sameHostsNumberOfRuns']
        );

        $data = array(
            'title' => $this->lang->line('mission_portal_title') . " - Preferences",
            'message' => $feedback,
            'is_displayable' => $this->ion_auth->is_accessible()
        );

        $data = array_merge($form_data, $data);
        $this->template->load('template', 'appsetting/missionportalpref', $data);
    }

    /**
     * updates the setting data
     */
    function _updateData($currentSettings = array())
    {
        $returnData = array();
        $returnData['status'] = false;
        $returnData['message'] = 'Data update failed';

        $blueHostHorizon = set_value('bluehost_threshold_global') * 60;

        $data                = array();
        $data['rbacEnabled'] = (bool)set_value('rbac');
        $data['logLevel'] = $this->input->post('api_log_level');
        $mode = set_value('mode');

        $data['blueHostHorizon'] = $blueHostHorizon; // store it in seconds
        $data['sameHostsNumberOfRuns'] = (int) $this->input->post('sameHostsNumberOfRuns');

        try {
            log_message('debug', "_updateData(): Trying to update data via REST ");
            $this->settings_rest_model->updateData($data);
            $returnData['status']  = true;
            $returnData['message'] = 'Data saved successfully.';
        } catch (Exception $e) {
            log_message('debug', "_updateData(): REST - Could not save your settings");
            $returnData['status'] = false;
            $returnData['message'] = "Could not save your settings. " . $e->getMessage();
        }
        return $returnData;
    }


    function required_valid_role($value)
    {
        if ($value == "select") {
            $this->form_validation->set_message('required_valid_role', 'A valid role is required.');
            return false;
        } else {
            return true;
        }
    }

    function validate_ldap_search($value)
    {
        $pattern = "/^\((.*)\)$/"; //  regex looks for start and end with parentheses

        if ($value && !preg_match($pattern, $value)) {
            $this->form_validation->set_message('validate_ldap_search', 'The search term should be enclosed in parentheses e.g \'(objectClass=inetOrgPerson)\'');
            return false;
        }

        return true;
    }

    function validate_bluehost_threshold($value)
    {
        $value = trim($value);
        if (empty($value)) {
            $this->form_validation->set_message('validate_bluehost_threshold', 'Unreachable host threshold field is required.');
            return false;
        }

        if (!ctype_digit($value)) {
            $this->form_validation->set_message('validate_bluehost_threshold', 'Unreachable host threshold field must contain a valid number of minutes.');
            return false;
        }

        if ($value < 5 || $value > 50400) {
            $this->form_validation->set_message('validate_bluehost_threshold', 'Unreachable host threshold field must be in range of 5 minutes - 5 weeks (50400 minutes).');
            return false;
        }

        return true;
    }

    function __db_check()
    {
        if ($this->input->post('mode') && strtolower($this->input->post('mode')) == 'internal') {
            return '|required';
        } else {
            return '';
        }
    }

    function get_native_roles()
    {
        $roles = $this->ion_auth->get_roles();
        $rolesarr = array();

        foreach ($roles as $role) {
            $rolesarr[$role['name']] = $role['name'];
        }
        $formData = form_dropdown('admin_role', $rolesarr);
        respond_ok($formData);
    }

    //for setting up usertime zone on fly.
    function setUserTimeZone()
    {
        $timezone = $this->input->post('userTimeZone');
        $this->session->set_userdata('user_timezone', $timezone);
    }

    function systemIdentifier($filter = 'sys')
    {
        $username = $this->session->userdata('username');
        $isAdmin = $this->ion_auth->is_admin();
        $this->load->model('advancedreports_model');
        $this->load->config('hostidentifier');
        $this->advancedreports_model->setRestClient($this->getRestClient());
        if (!$isAdmin) {
            redirect('auth/permission_deny', 'refresh');
        }

        $data = array(
            'title' => $this->lang->line('mission_portal_title') . " - Host identifier",
            "is_displayable" => $this->ion_auth->is_accessible()
        );
        // these variables will be execluded from the listing
        $includeVariables = config_item('host-identifier');
        if (empty($includeVariables)) {
            $includeVariables = array(
                'fqhost' => 'Fully qualified domain name'
            ); // Add one by default if no items are configured

        }

        $includeVariablesKeys = array_keys($includeVariables);
        $variableNames = array();
        foreach ($includeVariablesKeys as $keys) {
            $keysArray = explode('.', $keys);
            $variablesNames[] = $keysArray[1];
        }

        $in = "'" . implode("','", $variablesNames) . "'";
        $data['defaultList'] = $includeVariables;
        $data['exampleValue'] = array();

        $data['updated_host_identifier'] = $this->settings_rest_model->app_settings_get_item('hostIdentifier');
        // variable has prefix default. which we should delete, because updateHostIdentifier supports only scope and name
        if (!empty($data['updated_host_identifier'])) {
            $data['updated_host_identifier'] = str_replace('default.', '', $data['updated_host_identifier']);
        } else {
            $data['updated_host_identifier'] = 'sys.fqhost';
        }

        $query = sprintf('SELECT DISTINCT Bundle, VariableName, VariableValue FROM Variables');
        $query = $query . " WHERE VariableName IN ($in) AND hostkey in (SELECT hostkey from contexts where  ContextName = 'policy_server') ORDER BY VariableName";
        try {
            $reported = $this->advancedreports_model->runQuery($username, $query);
            $data['vars'] = $reported['rows'];
            foreach ((array)$data['vars'] as $row) {
                $key = $row[0] . '.' . $row[1];
                $value = $row[2];
                $data['exampleValue'][$key] = $value;
            }
        } catch (Exception $e) {
            log_message('error', "Error getting variables for system identifier:: " . $e->getMessage());
            $data['vars'] = array();
        }
        $this->template->load('template', 'appsetting/systemidentifier', $data);
    }

    function about()
    {

        $this->load->model('license_model');
        $this->license_model->setRestClient($this->ion_auth->getRestClient());
        $data = array();

        try {
            $licenseDetails = $this->license_model->getLicenseDetails();
            $data = array(
                'title' => $this->lang->line('mission_portal_title') . " - About CFEngine ",
                "is_displayable" => $this->ion_auth->is_accessible(),
                'details' => $licenseDetails,
                'licenseGranted' => isset($licenseDetails['granted']) ? $licenseDetails['granted'] : '',

                'licenseOwner' => isset($licenseDetails['owner']) ? $licenseDetails['owner'] : '',
                'licenseType' => isset($licenseDetails['licenseType']) ? $licenseDetails['licenseType'] : '',
                'expiry' => isset($licenseDetails['expires']) ? $licenseDetails['expires'] : ''
            );
        } catch (Exception $e) {
            log_message('license error:' . $e->getMessage());
        }

        $this->template->load('template', 'appsetting/about', $data);
    }

    function updateSystemIdentifierSetting()
    {
        if ($this->input->post('update')) {
            $username = $this->session->userdata('username');
            $scope = $this->input->post('scope', true);
            $name = $this->input->post('name', true);
            try {

                if (trim($scope) == '' && trim($name) == '') {
                    // reset the identifier
                    $data = array('hostIdentifier' => '');
                    $result = $this->settings_rest_model->updateData($data);
                } else {
                    $hostIdentifier = 'default.' . $scope . '.' . $name;
                    $data = array('hostIdentifier' => $hostIdentifier);
                    $result = $this->settings_rest_model->updateData($data);
                }
                $this->output->set_status_header(200, 'Update successful');
                $array = array(
                    'message' => 'Sucessfully updated',
                    'type' => 'success'
                );
                $this->output->set_output(json_encode($array));

            } catch (Exception $e) {
                $msg = $e->getMessage();
                $this->output->set_status_header(406, 'Update unsuccessful');
                $array = array(
                    'message' => $msg,
                    'type' => 'error'
                );
                $this->output->set_output(json_encode($array));
            }
        }
    }

    function designCenter()
    {

        if (!$this->ion_auth->is_accessible()) {
            redirect('settings/index');
        }

        $this->load->model('settings_model');
        $this->load->model('designcenter/dc_rest_model');

        $this->config->load('designcenter');
        $DcConfiguration['clonePath'] = get_git_clone_path();
        $DcConfiguration['keyPath'] = get_dc_user_ssh_key_directory();
        $DcConfiguration['privateKeyName'] = get_dc_user_ssh_key_name();
        $username = $this->session->userdata('username');
        $isAdmin = $this->ion_auth->is_admin();
        try {
            $userSettings = $this->dc_rest_model->get_settings();
        } catch (Exception $e) {
            $userSettings = null;
        }
        $feedback = array();

        if ($this->input->post('update')) {
            try {
                $feedback = $this->_handleFormUploadForGitSettings($DcConfiguration, $userSettings);
            } catch (Exception $e) {
                $output = $e->getMessage();
                $feedback = array(
                    'type' => 'alert-error',
                    'content' => $output
                );
            }

            if ($feedback === true) {
                redirect('settings/designCenter');
            }
        }

        $form_data = array(
            'gitServer' => set_value('gitServer', element_object('GIT_URL', $userSettings, NULL)),
            'gitEmail' => set_value('gitEmail', element_object('GIT_EMAIL', $userSettings, NULL)),
            'gitAuthor' => set_value('gitAuthor', element_object('GIT_AUTHOR', $userSettings, NULL)),
            'gitBranch' => set_value('gitBranch', element_object('GIT_BRANCH', $userSettings, NULL)),
            'gitPrivateKey' => set_value('gitPrivateKey', element_object('PKEY', $userSettings, NULL)),
        );
        $data = array(
            'title' => $this->lang->line('mission_portal_title') . " - Design center",
            'message' => $feedback,
            "is_displayable" => $this->ion_auth->is_accessible(),
            "hasSettings" => element_object('GIT_URL', $userSettings, NULL)
        );

        $data = array_merge($form_data, $data);

        $this->template->load('template', 'appsetting/design_center', $data);
    }

    function designCenter_reset()
    {

        if (!$this->ion_auth->is_accessible()) {
            redirect('settings/index');
        }

        $this->load->model('settings_model');
        $this->load->model('designcenter/dc_rest_model');

        $this->config->load('designcenter');
        try {
            $result = $this->dc_rest_model->delete_settings(); // its admin setting


        } catch (Exception $e) {
            $this->session->set_flashdata('message', array(
                'content' => "There was some error deleting git configuration",
                'type' => 'alert-error'
            ));
        }

        redirect('settings/designCenter');
    }

    function _handleFormUploadForGitSettings($dcConfiguration, $userSettings)
    {
        $this->form_validation->set_rules('gitServer', 'git server url', 'xss_clean|trim|required');
        $this->form_validation->set_rules('gitEmail', 'committer email', 'xss_clean|valid_email|trim|required');
        $this->form_validation->set_rules('gitAuthor', 'committer name', 'xss_clean|trim|required');
        $this->form_validation->set_rules('gitBranch', 'git branch', 'xss_clean|trim');
        if (empty($_FILES['gitPrivateKey']['name'])) {
            $this->form_validation->set_rules('gitPrivateKey', 'Git private key', 'required');
        }

        if ($this->form_validation->run() === true) {
            try {
                $postData = array(
                    'gitServer' => $this->input->post('gitServer', true),
                    'gitEmail' => $this->input->post('gitEmail', true),
                    'gitBranch' => $this->input->post('gitBranch', true),
                    'gitAuthor' => $this->input->post('gitAuthor', true),
                    "gitPrivateKey" => $_FILES["gitPrivateKey"]["tmp_name"]
                );
                $output = $this->dc_rest_model->create_settings($postData);
                $this->session->set_flashdata('message', array(
                    'content' => "The git repository has been successfully configured.",
                    'type' => 'alert-success'
                ));
                $feedback = true;
            } catch (Exception $e) {
                $message = json_decode($e->getMessage(), true);
                log_message('error', $message);
                $feedback = array(
                    'type' => 'alert-error',
                    'content' => $message
                );
            }
        } else {
            $feedback = validation_errors();
        }

        return $feedback;
    }

    function online_users()
    {

        $data = array(
            'title' => $this->lang->line('mission_portal_title') . " - Logged in users",
            "is_displayable" => $this->ion_auth->is_accessible()
        );

        $this->load->model('Authentication_model');

        $data['activeSessions'] = $this->Authentication_model->getActiveSession();

        $this->template->load('template', 'appsetting/users_online', $data);
    }

    function revoke_token($tokens)
    {
        $tokens = urldecode($tokens);
        $tokens = explode('|', $tokens);
        $this->load->model('Authentication_model');
        try {
            $this->Authentication_model->unsetToken($tokens);
        } catch (Exception $e) {
            log_message('error', 'ERROR revoking auth tokens. Error:: ' . $e->getMessage());
            show_error('Unexpected error while performing token revoke.');
        }
        redirect('settings/online_users');
    }

    function notification_scripts($action = null, $id = null)
    {
        !$this->ion_auth->is_accessible() && redirect('settings/index');

        if (!empty($action) && ($action == "edit" || $action == "add")) {
            redirect('settings/notification_scripts_update/' . $id);
        } else if (!empty($action) && ($action == "delete")) {
            redirect('settings/notification_scripts_delete/' . $id);
        }


        $data = array(
            'title' => $this->lang->line('mission_portal_title') . " - Custom notification scripts",
            "is_displayable" => $this->ion_auth->is_accessible()
        );

        $this->load->model('dashboard/alerts_model');

        try {
            $data['scripts'] = $this->alerts_model->getAllNotificationScripts();
        } catch (Exception $e) {
            log_message('error', 'Error getting scripts:: ' . $e->getMessage());
            show_error('Unexpected error while retrieving list of custom scripts.');
        }

        $this->template->load('template', 'appsetting/notification_scripts', $data);
    }


    function notification_scripts_delete($id)
    {
        !$this->ion_auth->is_accessible() && redirect('settings/index');

        if (!$this->ion_auth->is_accessible()) {
            redirect('settings/index');
        }

        $this->load->model('dashboard/alerts_model');
        $this->alerts_model->deleteAlertScript($id);
        redirect('settings/notification_scripts');

    }

    function notification_scripts_update($id = null)
    {
        !$this->ion_auth->is_accessible() && redirect('settings/index');

        $this->load->model('dashboard/alerts_model');

        $data = array(
            'title' => $this->lang->line('mission_portal_title') . " - Custom notification scripts",
            "is_displayable" => $this->ion_auth->is_accessible()
        );

        $feedback = false;
        $script = null;

        try {
            if (!empty($id)) {
                $script = $this->alerts_model->getNotificationScript($id);
            }
        } catch (Exception $e) {
            log_message('error', 'Error getting scripts:: ' . $e->getMessage());
            show_error('Unexpected error while retrieving details.');
        }

        $hasObjectValue = !empty($script);
        $data['script'] = $script;

        if ($this->input->post('update')) {
            try {
                $feedback = $this->_handleFormUploadForNotificationScripts($script);
            } catch (Exception $e) {
                $output = $e->getMessage();
                $feedback = array(
                    'type' => 'alert-error',
                    'content' => $output
                );
            }

            if ($feedback === true) {
                redirect('settings/notification_scripts');
            }
        }

        $formData = array(
            'scriptname' => set_value('scriptname', $hasObjectValue ? $script->getName() : null),
            'scriptdescription' => set_value('scriptdescription', $hasObjectValue ? $script->getDescription() : null),
            'scriptfilename' => set_value('scriptfilename', $hasObjectValue ? $script->getScriptFileName() : null),
            'id' => set_value('id', $hasObjectValue ? $script->getId() : null)
        );

        $data['message'] = $feedback;
        $data['form_data'] = $formData;

        $this->template->load('template', 'appsetting/notification_scripts_update', $data);

    }

    function _handleFormUploadForNotificationScripts($script)
    {

        $feedback = true;
        $this->form_validation->set_rules('scriptname', 'name', 'xss_clean|trim|required');
        $this->form_validation->set_rules('scriptdescription', 'description', 'xss_clean|trim');
        if (empty($script) && empty($_FILES['userfile']['name'])) {
            $this->form_validation->set_rules('userfile', 'file', 'required');
        }

        if ($this->form_validation->run() === true) {
            try {
                $data['name'] = $_POST['scriptname'];
                $data['description'] = $_POST['scriptdescription'];
                $uploadOk = true;
                if (!empty($_FILES['userfile']['name'])) {
                    //try upload the files

                    $config['upload_path'] = $this->config->item('notification_script_dir');;
                    $config['allowed_types'] = '*';

                    if (!file_exists($config['upload_path'])) {
                        if (!mkdir($config['upload_path'], 0755)) {
                            throw new Exception("Cannot make directory on path: " . $config['upload_path']);
                        }

                    }

                    $this->load->library('upload', $config);


                    if (!$this->upload->do_upload()) {
                        $feedback = array(
                            'type' => 'alert-error',
                            'content' => $this->upload->display_errors()
                        );
                        $uploadOk = false;
                    } else {
                        $fileData = $this->upload->data();
                        $data['scriptFileName'] = $fileData['file_name'];
                        chmod($fileData['full_path'], 0750); // make it executable
                    }

                }
                if ($uploadOk) {
                    if ($script) {
                        $this->alerts_model->updateAlertScripts($script, $data);
                    } else {
                        $this->alerts_model->insertAlertScripts($data);
                    }
                }

            } catch (Exception $e) {
                $message = $e->getMessage();
                log_message('error', $message);
                $feedback = array(
                    'type' => 'alert-error',
                    'content' => $message
                );
            }
        } else {
            $feedback = validation_errors();
        }

        return $feedback;
    }


    public function mail()
    {
        $isAdmin = $this->ion_auth->is_admin();
        if (!$isAdmin) {
            redirect('auth/permission_deny', 'refresh');
        }
        $this->load->model('MailSettingsModel');
        $settings = $this->MailSettingsModel->getSettings();
        $data = array(
            'title' => $this->lang->line('mission_portal_title') . " - Mail settings CFEngine ",
            "is_displayable" => $this->ion_auth->is_accessible(),
        );
        $data = array_merge($settings, $data);


        if ($this->input->method() == 'post') {
            $data = array_merge($data, $this->input->post());

            $this->form_validation->set_rules('protocol', 'Mail protocol', 'required');
            $this->form_validation->set_rules('max_email_attach_size', 'Max email attachment size (MB)', 'required|integer|less_than_equal_to[100]');
            $this->form_validation->set_rules('default_from_email', 'Default from email', 'required|valid_email');

            if ($this->input->post('protocol') == 'smtp') {
                $this->form_validation->set_rules('smtp_host', 'SMTP host', 'required');
                $this->form_validation->set_rules('smtp_user', 'SMTP user', 'required');
                $this->form_validation->set_rules('smtp_port', 'SMTP port', 'required');
            }

            if ($this->form_validation->run($this) == true) {
                $this->MailSettingsModel->updateSettings($this->input->post());
                if ($this->input->is_ajax_request()) {
                    return true;
                }
            } else {
                $errors = validation_errors();
                if ($this->input->is_ajax_request()) {
                    $this->output->set_status_header(419, 'validation error');
                    echo $errors['content'];
                    return false;
                }
                $this->session->set_flashdata('message', validation_errors());
            }
        }


        $this->template->load('template', 'appsetting/mail', $data);
    }

    public function sendTestEmail()
    {
        $isAdmin = $this->ion_auth->is_admin();
        if (!$isAdmin) {
            redirect('auth/permission_deny', 'refresh');
        }

        $this->load->model('MailSettingsModel');
        $defaultEmail = $this->MailSettingsModel->getSetting('default_from_email');

        $data = array(
            'title' => $this->lang->line('mission_portal_title') . " - Send test email ",
            'is_displayable' => $this->ion_auth->is_accessible(),
            'defaultEmail' => $defaultEmail,
        );

        $this->form_validation->set_rules('from', 'From', 'required|valid_email');
        $this->form_validation->set_rules('to', 'To', 'required|valid_email');

        if ($this->input->post() && $this->form_validation->run($this) == true) {
            $this->load->library('email');
            $this->email->from($this->input->post('from'));
            $this->email->to($this->input->post('to'));
            $this->email->subject('CFEngine Enterprise hub test email');
            $this->email->message('Test email sent from ' . gethostname() . ' by user ' . $this->session->userdata('username') . '.');
            $result = $this->email->send();
            if (!$result) {
                $data['debugInfo'] = $this->email->print_debugger();
                log_message('debug', 'send test email error:' . $data['debugInfo']);
            } else {
                $data['debugInfo'] = 'Email successfully sent';
            }
        } else {
            $this->session->set_flashdata('message', validation_errors());
        }
        $data = array_merge($data, $this->input->post());
        $this->template->load('template', 'appsetting/sendTestEmail', $data);
    }

    public function ldap_api_settings()
    {
        $isAdmin = $this->ion_auth->is_admin();
        if (!$isAdmin) {
            redirect('auth/permission_deny', 'refresh');
        }

        $ldapApiUrl = $this->config->item('ldapApiUrl');
        $pestClient = new Pest($ldapApiUrl);
        $apiServerLDAPSettings = $this->ldap_api->getSettings($pestClient);
        $is_ldap_available = $this->ldap_api->testLdapUrl($pestClient);

        $settingsValue = $this->settings_rest_model->get_app_settings();

        if (!$is_ldap_available) {
            log_message('error', 'Could not communicate with LDAP API at ldapApiUrl=' . $ldapApiUrl . ' check configuration in /var/cfengine/share/GUI/application/config/appsettings.php');
        }

        $data = array(
            'title' => $this->lang->line('mission_portal_title') . ' - Authentication settings',
            'is_displayable' => $this->ion_auth->is_accessible(),
            'is_correct_api_url' => (bool)$apiServerLDAPSettings,
            'is_ldap_available' => $this->ldap_api->testLdapUrl($pestClient),
            'mode' => $settingsValue['ldapEnabled'] ? 'ldap' : 'internal'
        );

        $this->form_validation->set_rules('domain_controller', 'Ldap Host', 'required');
        $this->form_validation->set_rules('port', 'Port', 'required|integer');
        $this->form_validation->set_rules('base_dn', 'Base DN', 'required');
        $this->form_validation->set_rules('login_attribute', 'Login attribute', 'required');
        $this->form_validation->set_rules('ldap_filter', 'Ldap filter', 'trim');
        $this->form_validation->set_rules('admin_username', 'LDAP bind user name', 'required');
        $this->form_validation->set_rules('admin_password', 'LDAP bind password', 'required');

        if ( $this->input->post() ) {
            if ( $this->form_validation->run( $this ) == true ) {
            $response = $this->ldap_api->saveSettings($pestClient, $this->input->post());
            if (!$response['success']) {
                $message = [
                    'content' => ucfirst($response['message']),
                    'type' => 'alert-error'
                ];
            } else {
                $message = [
                    'content' => 'Settings were successfully saved.',
                    'type' => 'alert-success'
                ];
            }
            $apiServerLDAPSettings = $this->input->post();
            $this->session->set_flashdata('message', $message);
            } else {
                $apiServerLDAPSettings = $this->input->post();
                $this->session->set_flashdata( 'message', validation_errors() );
            }
        }


        $data = array_merge($apiServerLDAPSettings, $data);
        $data['admin_password'] = isset($data['admin_password']) && $data['admin_password'] != '' ? 'Password is set' : 'Password is not set';
        $this->template->load('template', 'appsetting/auth_settings', $data);
    }

    public function saveAuthenticationMethod()
    {
        $returnData = $data = [];
        if ((!empty($this->input->post('mode')))) {
            $data['ldapEnabled'] = boolval($this->input->post('mode') == 'ldap');
        }

        try {
            log_message('debug', "saveAuthenticationMethod(): Trying to update data via REST ");
            $this->settings_rest_model->updateData($data);
            $returnData['status'] = true;
            $returnData['message'] = 'Data saved successfully.';
        } catch (Exception $e) {
            log_message('debug', "saveAuthenticationMethod(): REST - Could not save your settings");
            $returnData['status'] = false;
            $returnData['message'] = "Could not save your settings. " . $e->getMessage();
        }
        echo json_encode($returnData);
    }
}
